#
# Makefile Template for Tessy
# Compiler: GNU Tools for ARM
# Debugger: LAUTERBACH TRACE32
# Template Version: 2
#

# Defines to be generated by Tessy:
#
# BSP_DIRECTORYECTORY   :=
# BUFFER_SIZE           :=
# COMPILER_DEFINES      :=
# COMPILER_INCLUDES     :=
# COMPILER_INSTALL_PATH :=
# EABI                  :=
# ENDIANESS             :=
# EXEFILE_EXTENSION     :=
# MODULE_PATH           :=
# MODULE_PATH_DOS       :=
# OBJFILE_EXTENSION     :=
# PROJECTROOT           :=
# TARGET_INSTALL_PATH   :=
# TESSY_BIN             :=
# TESSY_SYS             :=
# TESTOBJECT            :=
# TIMER_FILE            :=
# USER_LINK_OPTIONS     :=

#####################################################################
#BEGIN_DEFINES
#####################################################################

#
# Special macros for includes and defines
#
TS_DEF_SEPARATOR :=-D
define TS_DEFINE
$(subst ;, $(TS_DEF_SEPARATOR),$(1))
endef

TS_INC_SEPARATOR :=-I
define TS_INCLUDE
$(subst ;, $(TS_INC_SEPARATOR),$(1))
endef

#
# Special macros for the linker's arguments file
#
MODULE_PATH_DOS := $(shell ts_pathname -s '$(MODULE_PATH_DOS)')
ARGS_FILE       := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s.args

define TS_LONGPATH
"$(shell ts_pathname -w '$(1)' | sed 's/\\\\/\\//g')"
endef

define TS_WRITE_LINE
echo $(call TS_LONGPATH,$(1),$(ARGS_FILE)) >> "$(ARGS_FILE)"

endef

define TS_WRITE_LINKER_FILE
$(foreach var,$(1),$(call TS_WRITE_LINE,$(var)))
endef

define TS_CREATE_ARGS_FILE
$(file >$(ARGS_FILE))
endef

#
# Short pathes
#
TESSY_SYS_DOS   := $(shell ts_pathname -s $(TESSY_SYS))
TESSY_SYS       := $(shell ts_pathname -s $(TESSY_SYS))
GNU_TOOLS_HOME  := $(shell ts_pathname -s '$(COMPILER_INSTALL_PATH)')
MODULE_PATH     := $(shell ts_pathname -s '$(MODULE_PATH)')
LIBRARY         := $(shell ts_pathname -s '$(LIBRARY)')

TS_NAME         := TS_TRACE32
IDE_NAME        := trace32
COMPANY_NAME    := gnutools
PROCESSOR_NAME  := arm

ifeq ("little","$(ENDIANESS)")
TS_BYTEORDER := TS_HOST_BYTEORDER
TS_BYTEORDER_OPTION :=
endif
ifeq ("big","$(ENDIANESS)")
TS_BYTEORDER := TS_NET_BYTEORDER
TS_BYTEORDER_OPTION :=
endif
ifndef TS_BYTEORDER
$(error Attribute 'Endianess' has invalid value '$(ENDIANESS)', only 'little' and 'big' are allowed.)
endif

ifeq ($(ALIGN_COMM_BUFFER),)
ALIGN_COMM_BUFFER := 0
endif

all	 : master slave
master : $(MODULE_PATH)/ts_$(TESTOBJECT)_m.exe
slave  : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(EXEFILE_EXTENSION)

#
# MASTER & COMM Makefiles
# (you should not need to modify these settings)
#
M_COMM_MAKE		:=	$(TESSY_SYS)/make/master_$(IDE_NAME)_gnu_i386.mk
S_COMM_MAKE		:=	$(TESSY_SYS)/src/comm/comm_$(IDE_NAME)_$(COMPANY_NAME)_$(PROCESSOR_NAME).mk

#
# SLAVE
# (You may change these settings to use your own compiler/linker options)
#
S_CC           := $(shell ts_pathname -s '$(GNU_TOOLS_HOME)')\bin\arm-$(EABI)-gcc.exe
S_LINK         := $(shell ts_pathname -s '$(GNU_TOOLS_HOME)')\bin\arm-$(EABI)-gcc.exe
S_OBJCOPY      := $(shell ts_pathname -s '$(GNU_TOOLS_HOME)')\bin\arm-$(EABI)-objcopy.exe
S_INCLUDES     := -I"$(TESSY_SYS_DOS)\include\tessy\comm" -I"$(MODULE_PATH_DOS)"
S_STUB_OBJECT  := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs$(OBJFILE_EXTENSION)
S_UC_OBJECT    := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr$(OBJFILE_EXTENSION)
# Tested CPU = cortex-m0
# Tested board defines = -DUSE_NUCLEO_F030R8 -DHSI_VALUE=8000000 -DSTM32F0XX_VL -DUSE_STDPERIPH_DRIVER
S_COMP_OPTIONS := -c -mthumb -mcpu=$(CPU) $(COMPILER_DIALECT) $(call TS_DEFINE,$(COMPILER_DEFINES)) \
                  -DTESSY -DTS_SLAVE -D$(TS_NAME) -DTS_HAVE_INT64 -DTS_HAVE_FLOAT64 -O0 -ffunction-sections \
		  -fdata-sections -gdwarf -g -Wall
S_ASM_OPTIONS  := -c -mthumb -mcpu=$(CPU) -g -Wa,--warn -x assembler-with-cpp
S_LINK_OPTIONS := -mthumb -mcpu=$(CPU) -gdwarf -Wl,--start-group -lc -lm -Wl,--end-group -static $(if $(LINKER_FILE),-T "$(LINKER_FILE)") \
                  -Wl,-cref,-u,Reset_Handler -Wl,--gc-sections -Wl,--defsym=malloc_getpagesize_P=0x1000

#####################################################################
#END_DEFINES
#####################################################################

#####################################################################
#BEGIN_SOURCE
#####################################################################

#
# Template rules
#
$(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER)$(OBJFILE_EXTENSION): $(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER).c
	$(S_CC) $(S_INCLUDES) $(call TS_INCLUDE,$(USER_INCLUDES$$(SOURCENUMBER))) $(S_COMP_OPTIONS) \
		$(call TS_DEFINE,$(USER_DEFINES$$(SOURCENUMBER))$(S_COM_OPTIONS)) \
		$(USER_COMP_OPTIONS$$(SOURCENUMBER)) -o "$@" "$<"


#####################################################################
#END_SOURCE
#####################################################################

#####################################################################
#BEGIN_TARGETS
#####################################################################

# COMPILE USERCODE / STUBS
#
$(S_UC_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr.c
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o "$@" "$<"

$(S_STUB_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs.c
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o "$@" "$<"


#
# COMPILE/LINK MASTER
#

include $(M_COMM_MAKE)


#
# COMPILE SLAVE COMMUNICATION MODULES

include $(S_COMM_MAKE)


#
#
# COMPILE/LINK SLAVE
#
$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s.c
	@echo *
	@echo ***** Compiling Slave *****
	$(S_CC) $(S_COMP_OPTIONS) $(S_INCLUDES) -o "$@" "$<"

#
# Handle Board Files
#
ifeq ("true","$(USE_BOARD_FILES)")
ifeq ("","$(INITOBJDIR)")
$(error Attribute 'Use Board Files' is set to true but 'InitObjDir' is empty.)
endif
INITOBJDIR := $(shell ts_pathname -s '$(INITOBJDIR)')
ifeq ("","$(INITSRCDIR)")
$(info Attribute 'InitSrcDir' is empty. Using objects for linking only.)
START_OBJS := $(wildcard $(INITOBJDIR)/*$(OBJFILE_EXTENSION))
else
INITSRCDIR := $(shell ts_pathname -s '$(INITSRCDIR)')
START_OBJS := $(patsubst $(INITSRCDIR)/%.c,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.c)) \
              $(patsubst $(INITSRCDIR)/%.s,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.s))

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.c
	$(S_CC) $(S_COMP_OPTIONS) $(S_INCLUDES) -I$(INITSRCDIR) "$<" -o"$@"

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.s
	$(S_CC) $(S_COMP_OPTIONS) $(S_INCLUDES) "$<" -o"$@"
endif
ifeq ("","$(START_OBJS)")
$(warning Attribute 'Use Board Files' is set to true but no objects were found.)
endif
endif

#
# Objects to be delete after the build process.
#
TS_INTERMEDIATE_OBJS := $(S_OBJECTS) $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) \
                        $(S_SRC_OBJECTS) $(S_UC_OBJECT) $(S_STUB_OBJECT)
.INTERMEDIATE: $(TS_INTERMEDIATE_OBJS)

$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(EXEFILE_EXTENSION) : $(START_OBJS) $(TS_INTERMEDIATE_OBJS)
	@echo *
	@echo ***** Linking Slave *****
	$(call TS_CREATE_ARGS_FILE)
	@$(call TS_WRITE_LINKER_FILE,$(TS_INTERMEDIATE_OBJS))
ifneq ($(START_OBJS),)
	@$(call TS_WRITE_LINKER_FILE,$(START_OBJS))
endif
	$(S_LINK) $(S_LINK_OPTIONS) $(USER_LINK_OPTIONS) \
		-Wl,-Map="$(MODULE_PATH_DOS)\ts_$(TESTOBJECT).map" \
		@"$(ARGS_FILE)" \
		-o "$@" $(LIBRARY)

#####################################################################
#END_TARGETS
#####################################################################

